package nl.cwi.sen1.tunit;

import java.net.InetAddress;

import toolbus.ToolBus;
import toolbus.adapter.AbstractTool;
import aterm.ATerm;

public abstract class ToolComValidator extends AbstractTool implements Runnable{
	private volatile TestToolBridge testToolBridge;
	private final TUnitTestCase testCase;
	
	private final String name;
	private final int id;
	
	private final boolean verbose;
	
	public ToolComValidator(TUnitTestCase testCase, String name, boolean verbose){
		super();
		
		this.testCase = testCase;
		
		this.name = name;
		this.id = -1;
		
		this.verbose = verbose;
	}
	
	public ToolComValidator(TUnitTestCase testCase, String name, int id, boolean verbose){
		super();
		
		this.testCase = testCase;
		
		this.name = name;
		this.id = id;
		
		this.verbose = verbose;
	}
	
	public void connect(ToolBus toolbus) throws Exception{
		connectDirectly(toolbus, Thread.currentThread().getContextClassLoader(), name, id);
	}
	
	/**
	 * @see toolbus.adapter.AbstractTool#connect(String[])
	 */
	public void connect(String[] args) throws Exception{
		String toolName = null;
		int toolID = -1;

		InetAddress host = null;
		int port = -1;

		for(int i = 0; i < args.length; i++){
			String arg = args[i];
			if(arg.equals("-TB_TOOL_NAME")){
				toolName = args[++i];
			}else if(arg.equals("-TB_TOOL_ID")){
				toolID = Integer.parseInt(args[++i]);
			}else if(arg.equals("-TB_HOST")){
				host = InetAddress.getByName(args[++i]);
			}else if(arg.equals("-TB_PORT")){
				port = Integer.parseInt(args[++i]);
			}
		}

		if(toolName == null) throw new RuntimeException("Missing tool identification.");

		testToolBridge = new TestToolBridge(testCase, termFactory, this, toolName, toolID, host, port, verbose);
		toolBridge = testToolBridge;
		toolBridge.run();
	}
	
	public void connectDirectly(ToolBus toolbus, ClassLoader toolClassLoader, String toolName, int toolID) throws Exception{
		if(toolName == null) throw new RuntimeException("Missing tool identification.");

		testToolBridge = new TestToolBridge(testCase, termFactory, this, toolName, toolID, toolClassLoader, toolbus, verbose);
		toolBridge = testToolBridge;
		toolBridge.run();
	}
	
	public void sendEvent(ATerm event){
		sendEvent(event, TestToolBridge.DEFAULTTIMEOUT);
	}
	
	public void sendEvent(ATerm event, long timeout){
		testToolBridge.sendEvent(event, timeout);
	}
	
	public ATerm sendRequest(ATerm request, ATerm expectedResponse){
		return sendRequest(request, expectedResponse, TestToolBridge.DEFAULTTIMEOUT);
	}
	
	public ATerm sendRequest(ATerm request, ATerm expectedResponse, long timeout){
		return testToolBridge.sendRequest(request, expectedResponse, timeout);
	}
	
	public void registerForDo(ATerm action){
		testToolBridge.registerForDo(action);
	}
	
	public void registerForEval(ATerm action, ATerm expectedResult){
		testToolBridge.registerForEval(action, expectedResult);
	}
	
	public void expectAction(){
		expectAction(TestToolBridge.DEFAULTTIMEOUT);
	}
	
	public void expectAction(long timeout){
		testToolBridge.expectAction(timeout);
	}
	
	public void waitForCompletion(){
		testToolBridge.waitForCompletion(TestToolBridge.DEFAULTTIMEOUT);
	}
	
	public void waitForCompletion(long timeout){
		testToolBridge.waitForCompletion(timeout);
	}
	
	public void disconnect(){
		disconnect(termFactory.makeList());
	}
	
	public void receiveAckEvent(ATerm aTerm){
		// Ignore
	}
	
	public void receiveTerminate(ATerm aTerm){
		// Ignore
	}
}
