package nl.cwi.sen1.tunit.execution;

import java.util.HashMap;
import java.util.Map;

import nl.cwi.sen1.tunit.TUnitTestCase;
import toolbus.ToolBus;
import toolbus.tool.ToolDefinition;
import toolbus.tool.ToolInstance;
import toolbus.tool.execution.DefaultToolExecutor;
import toolbus.tool.execution.IToolExecutor;
import toolbus.tool.execution.IToolExecutorFactory;

public class DefaultTestToolExecutorFactory implements IToolExecutorFactory{
	private final TUnitTestCase testCase;
	
	private final Map<String, Class<?>> execuableToolComValidators;
	
	public DefaultTestToolExecutorFactory(TUnitTestCase testCase){
		super();
		
		this.testCase = testCase;
		
		execuableToolComValidators = new HashMap<String, Class<?>>();
	}
	
	public void addComValidatorForExecutableTool(String name, Class<?> toolStubClass){
		execuableToolComValidators.put(name, toolStubClass);
	}
	
	public IToolExecutor getToolExecutor(ToolInstance toolInstance, ToolDefinition toolDefinition, ToolBus toolbus){
		String name = toolInstance.getToolName();
		Class<?> toolComValidatorClass = execuableToolComValidators.get(name);
		if(toolComValidatorClass != null){
			return new ToolComValidatorExecutor(testCase, name, toolInstance.getToolID(), toolComValidatorClass);
		}
		
		return new DefaultToolExecutor(toolInstance, toolDefinition, toolbus);
	}
}
