package nl.cwi.sen1.tunit.execution;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import nl.cwi.sen1.tunit.TUnitTestCase;
import nl.cwi.sen1.tunit.ToolComValidator;
import toolbus.exceptions.ToolBusException;
import toolbus.logging.ILogger;
import toolbus.logging.IToolBusLoggerConstants;
import toolbus.logging.LoggerFactory;
import toolbus.tool.execution.IToolExecutor;

public class ToolComValidatorExecutor implements IToolExecutor{
	private final TUnitTestCase testCase;
	
	private final String name;
	private final int id;
	private final Class<?> toolClass;
	
	public ToolComValidatorExecutor(TUnitTestCase testCase, String name, int id, Class<?> toolClass){
		super();
		
		this.testCase = testCase;
		
		this.id = id;
		this.name = name;
		this.toolClass = toolClass;
	}
	
	public void execute() throws ToolBusException{
		// Instantiate the toolstub.
		ToolComValidator tool;
		try{
			Constructor<?> toolConstructor = toolClass.getConstructor(new Class[]{TUnitTestCase.class, String.class, int.class, boolean.class});
			
			tool = (ToolComValidator) toolConstructor.newInstance(new Object[]{testCase, name, new Integer(id), Boolean.TRUE});
			testCase.connectToolComValidator(tool);
			
			Thread toolExecutor = new Thread(tool);
			toolExecutor.setDaemon(true);
			toolExecutor.start();
		}catch(InstantiationException iex){
			String error = "Unable to instantiate the tool. Classname: " + toolClass.getName();
			LoggerFactory.log(error, iex, ILogger.ERROR, IToolBusLoggerConstants.TOOLINSTANCE);
			throw new ToolBusException(error);
		}catch(IllegalAccessException iaex){
			String error = "The constructor of the tool we are trying to instantiate isn't public. Classname: " + toolClass.getName();
			LoggerFactory.log(error, iaex, ILogger.ERROR, IToolBusLoggerConstants.TOOLINSTANCE);
			throw new ToolBusException(error);
		}catch(InvocationTargetException itex){
			String error = "An exception occured during the invokation of the constructor of the tool we are trying to instantiate. Classname: " + toolClass.getName();
			LoggerFactory.log(error, itex, ILogger.ERROR, IToolBusLoggerConstants.TOOLINSTANCE);
			throw new ToolBusException(error);
		}catch(NoSuchMethodException nsmex){
			String error = "The tool we are trying to instantiate doesn't have a proper constructor. Classname: " + toolClass.getName();
			LoggerFactory.log(error, nsmex, ILogger.ERROR, IToolBusLoggerConstants.TOOLINSTANCE);
			throw new ToolBusException(error);
		}catch(SecurityException sex){
			String error = "We don't have permission to invoke the constructor of: " + toolClass.getName();
			LoggerFactory.log(error, sex, ILogger.ERROR, IToolBusLoggerConstants.TOOLINSTANCE);
			throw new ToolBusException(error);
		}catch(Exception ex){
			String error = "Unable to connect tool to the ToolBus directly: " + toolClass.getName();
			LoggerFactory.log(error, ex, ILogger.ERROR, IToolBusLoggerConstants.TOOLINSTANCE);
			throw new ToolBusException(error);
		}
	}
}
